<?php
/* Qr Crypto payment Gateway.
*
* Provides a Crypto Payment Gateway.
*
* @class       ECWP_Payment_Crypto
* @extends     WC_Payment_Gateway
* @version     1.0.0
* @package     WooCommerce\Classes\Payment
*/


class ECWP_Payment_Crypto extends WC_Payment_Gateway
{

    /**
     * Constructor for the gateway.
     */
    public function __construct()
    {
        // Setup general properties.
        $this->setup_properties();

        // Load the settings.
        $this->init_form_fields();
        $this->init_settings();

        // Get settings.
        $this->title = "eCredits";
        $this->description = "";
        $this->instructions = "Scan QR-code to pay";
        $this->stagging = $this->get_option('stagging');
        $this->storeId = $this->get_option('storeId');
        $this->segmentId = $this->get_option('segmentId');
        $this->secret = $this->get_option('secret');
        $this->clientId = $this->get_option('clientId');
        $this->webhookSecret = $this->get_option('webhookSecret');

        add_action('woocommerce_update_options_payment_gateways_' . $this->id, array($this, 'process_admin_options'));
        add_action('woocommerce_multistep_checkout_after_order_info', 'add_my_custom_step');
        add_action('woocommerce_thankyou_' . $this->id, array($this, 'thankyou_page'));
        // webhook
        add_action('woocommerce_api_e-credits_response', array($this, 'webhook'));
    }

    /**
     * Setup general properties for the gateway.
     */
    protected function setup_properties()
    {
        $this->id = 'crypto';
        //Here you can change title and description of plugin
        $this->method_title = __('eCredits', 'woocommerce');
        $this->method_description = __('Become part of the future today! Accept eCredits (ECS) in your onlineshop. The easy-to-use plugin allows you to offer your customers to
        checkout with ECS. Simple and quick installation. Learn more in our User
        Guide.', 'woocommerce');
        $this->has_fields = false;
    }

    /**
     * Initialise Gateway Settings Form Fields.
     */
    public function init_form_fields()
    {
        $this->form_fields = array(
            'enabled' => array(
                'title' => __('eCredits', 'woocommerce'),
                'label' => __('Enable eCredits payment', 'woocommerce'),
                'type' => 'checkbox',
                'description' => '',
                'default' => 'no',
            ),
            'stagging' => array(
                'title' => __('Test mode', 'woocommerce'),
                'label' => __('Enable test mode', 'woocommerce'),
                'type' => 'checkbox',
                'description' => '',
                'default' => 'no',
            ),
            'storeId' => array(
                'title' => __('Store Id', 'woocommerce'),
                'type' => 'text',
                'description' => __('Is the ID of your store in eCredits.', 'woocommerce'),
                'default' => __('12345', 'woocommerce'),
                'desc_tip' => true,
            ),
            'segmentId' => array(
                'title' => __('Segment Id', 'woocommerce'),
                'type' => 'text',
                'description' => __('If you offer eCashback, set the cashback segment.', 'woocommerce'),
                'default' => __('12345', 'woocommerce'),
                'desc_tip' => true,
            ),
            'secret' => array(
                'title' => __('Secret', 'woocommerce'),
                'type' => 'password',
                'description' => __('The generated API secret from the portal.', 'woocommerce'),
                'default' => __('12345', 'woocommerce'),
                'desc_tip' => true,
            ),
            'clientId' => array(
                'title' => __('Client Id', 'woocommerce'),
                'type' => 'text',
                'description' => __('The client ID is the same as ProfileID.', 'woocommerce'),
                'default' => __('12345', 'woocommerce'),
                'desc_tip' => true,
            ),
            'webhookSecret' => array(
                'title' => __('webhookSecret', 'woocommerce'),
                'type' => 'password',
                'description' => __('Define a secret for your webhook.', 'woocommerce'),
                'default' => __('12345', 'woocommerce'),
                'desc_tip' => true,
            ),
        );
    }

    /**
     * Process the payment and return the result.
     *
     * @param int $order_id Order ID.
     * @return array
     */
    public function process_payment($order_id)
    {
        $order = wc_get_order($order_id);

        if ($order->get_total() > 0) {
            $order->update_status(apply_filters('woocommerce_crypto_process_payment_order_status', $order->has_downloadable_item() ? 'on-hold' : 'pending-payment', $order), __('Waiting for qr payment.', 'woocommerce'));
        } else {
            $order->payment_complete();
        }
        // Remove cart.
        // WC()->cart->empty_cart();

        // Return thankyou redirect.
        return array(
            'result' => 'success',
            'redirect' => $this->get_return_url($order),
        );
    }

    /**
     * Insert in to transaction table.
     */
    function insertInToTable($order, $array)
    {
        // WP Globals
        global $table_prefix, $wpdb;

        // Customer Table
        $customerTable = $table_prefix . 'e_credits_transactions';
        $currentTime = time();
        $wpdb->insert($customerTable, array(
            "order_id" => (int)$order,
            "timestamp" => $currentTime,
            "transaction_id" => $array["id"],
            "qrCodeBase64" => $array["qrCodeBase64"],
            "expiresAt" => $array["expiresAt"],
            "total" => (float)$array["amountDue"],
            "totalCurrency" => $array["currencyCode"],
            "totalFiat" => (float)$array["fiatAmountDue"],
            "totalFiatCurrency" => $array["fiatCurrencyCode"]
        ));
    }

    /**
     * Check if order exist already
     */
    function order_exist($order)
    {
        // WP Globals
        global $table_prefix, $wpdb;

        // Customer Table
        $customerTable = $table_prefix . 'e_credits_transactions';

        $result = $wpdb->get_results(
            $wpdb->prepare("SELECT * FROM `$customerTable` WHERE order_id = %d ORDER BY id DESC LIMIT 1;", $order)
        );
        return $result;
    }

    /**
     * Api call to get qr code.
     */
    function generate_qr_code($payload)
    {
        //  API url
        if ($this->stagging == 'yes') {
            $url = "https://widget.api.tst.ecredits.com/api";
        } else {
            $url = "https://widget.api.ecredits.com/api";
        }

        $auth = [
            "secret" => $this->secret,
            "clientId" => $this->clientId
        ];
        $args = [
            'headers' => ['Content-Type'  => 'application/json'],
            'body'        => wp_json_encode($auth),
            'timeout' => 10,
            'method' => 'POST'
        ];
        $responseAuth = wp_remote_post($url . '/Authentication', $args);
        $responseArray = json_decode(wp_remote_retrieve_body($responseAuth), true);

        //Bearer token
        $token = $responseArray["token"];
        //Qr code call
        $args = [
            'headers' => ['Content-Type'  => 'application/json', 'Authorization' => 'Bearer ' . $token],
            'body'        => wp_json_encode($payload),
            'timeout' => 10,
            'method' => 'POST'
        ];
        $response = wp_remote_retrieve_body(wp_remote_post($url . '/v1/Requests', $args));
        return $response;
    }

    /**
     * Output for the order received page.
     */
    public function thankyou_page($order)
    {
        $orderDetails = wc_get_order($order);
        $payload = array(
            'currencyCode' => $orderDetails->data["currency"],
            'amountDue' => $orderDetails->data["total"],
            'storeId' => $this->storeId,
            'generateQrCode' => true,
            'paymentReference' => 'woocommerce',
        );
        //Check if order already exist in DB
        $orderExist = $this->order_exist($order);
        $newQr = strtotime("now") - strtotime($orderExist[0]->expiresAt) > 0;
        $status = $orderDetails->data["status"];
        if (isset($_POST["cancelled"])) {
            echo '<script type="text/javascript">',
            'location.assign("' . get_site_url() . "/cart" . '")',
            '</script>';
            $orderDetails->update_status(apply_filters('woocommerce_payment_complete_order_status', 'cancelled', $orderDetails), __('Order is cancelled.', 'woocommerce'));;
            $status = "cancelled";
        }

        if ((empty($orderExist) || count($orderExist) < 1 || $newQr) && $status == "pending") {
            //Api call
            $response = $this->generate_qr_code($payload);
            //String to object
            $responseArray = json_decode($response, true);

            //Inserting into wp table
            $this->insertInToTable($order, $responseArray);
            $qr = $responseArray["qrCodeBase64"];
            $time = strtotime($responseArray["expiresAt"]);
        } else {
            $qr = $orderExist[0]->qrCodeBase64;
            $time = strtotime($orderExist[0]->expiresAt);
        }
        $time = date("D M d Y H:i:s O", $time);
        //again Check if order already exist in DB
        $orderExist = $this->order_exist($order);

        //start JS timer
        echo '<script type="text/javascript">',
        'let x = timer("' . $time . '");',
        '</script>';

        if ($orderDetails->data["status"] !== "completed") {
            echo '<script type="text/javascript">',
            'orderStatus("' . $order . '", "' . get_site_url() . '");',
            '</script>';
        }
        if ($responseArray !== null || $orderExist) {
            echo '<div style="display: flex; align-items: center; flex-direction: column;">';
            //Display Qr code with instructions
            echo wp_kses_post(wpautop(wptexturize($this->instructions)));
            if ($status == "pending") {
                $paymentData = "";
                $paymentData .= '<p style="margin: 0;" id="timer-desc"></p>';
                $paymentData .= '<div id="amount" style="display: flex; justify-content: center;">';
                $paymentData .= '<div style="display: flex; flex-direction:column; align-items: center;">';
                $paymentData .= '<h5 style="margin: 0;">' .  esc_html($orderExist[0]->total) . '&nbsp;' . esc_html($orderExist[0]->totalCurrency) . ' </h5>';
                $paymentData .= '<p style="margin: 0;">' . "≈&nbsp;" . esc_html($orderExist[0]->totalFiat) . '&nbsp;' . esc_html($orderExist[0]->totalFiatCurrency) . ' </p>';
                $paymentData .= '</div>';
                $paymentData .= '</div>';
                $paymentData .= '</div>';
                $paymentData .= '<div id="qr" class="qr" style="display: flex; justify-content: center;">';
                $paymentData .= '<img style="width:auto; height:auto;" src="data:image/png;base64, ' . esc_attr($qr) . '">';
                $paymentData .= '</div>';
                $paymentData .= '<div style="display: flex; justify-content: center;">';
                $paymentData .= '<p id="timer"></p>';
                $paymentData .= '</div>';
                $paymentData .= '<div style="display: flex; justify-content: center;">';
                $paymentData .= '<button type="submit" style="margin-right: 10px; display: none;" onclick="location.reload()" id="timer-reset">Try again</button>';
                $paymentData .= '<form id="cancel" method="post">
                          <input type="hidden" name="cancelled" value="1">
                          <button type="submit" name="cancelled-btn">Cancel order</button>
                          </form>';
                echo $paymentData;
                echo '</div>';
            } else if ($status == "cancelled") {
                echo '<p>Order is cancelled</p>';
                echo '</div>';
            } else {
                echo '<p>Order is completed</p>';
                echo '</div>';
            }
        } else {
            echo '<p>' . esc_html($response) . '</p>';
        }
    }

    /**
     * Webhook.
     */
    public function webhook()
    {
        // WP Globals
        global $table_prefix, $wpdb;

        if (isset($_GET["check_order"])) {
            $orderId = (int)$_GET["check_order"];
            $order = wc_get_order($orderId);
            if ($order->data["status"] === "completed") {
                echo json_encode(["success" => "success"]);
                // Remove cart.
                WC()->cart->empty_cart();
            } else {
                echo json_encode(["error" => "error"]);
            }
            exit;
        }
        $entityBody = file_get_contents('php://input');
        try {
            $response = json_decode($entityBody, true);
            // Customer Table
            $customerTable = $table_prefix . 'e_credits_transactions';
            $id = sanitize_text_field($response["requestId"]);
            $result = $wpdb->get_results(
                $wpdb->prepare("SELECT * FROM `$customerTable` WHERE transaction_id = %s  LIMIT 1;", [$id])

            );
            $order = wc_get_order($result[0]->order_id);
            $signature_valid = base64_encode(hash_hmac('sha256', $id, $this->webhookSecret, true));
            if ($signature_valid === $response["signature"] && $response["status"] == 1) {
                $order->payment_complete();
                $order->update_status(apply_filters('woocommerce_crypto_process_payment_order_status', 'completed', $order), __('Payment with eCredits confirmed.', 'woocommerce'));
            }
        } catch (Exception $e) {
            echo "error";
        }
    }
}
